<?php
/**
 * Cache functions for news_client.
 */

/**
 * Page callback for admin/config/services/news_client/cache.
 */
function news_client_cache_page() {
  $last_cron = variable_get('news_client_last_cron', 0);
  $last_cron_text = ($last_cron > 0) ? format_date($last_cron) : 'Never';
  
  $last_update = variable_get('news_client_last_update', 0);
  $last_update_text = ($last_update > 0) ? format_date($last_update) : 'Never';
  
  $view = views_get_view('news_client_cache_view');
  
  // Need to change override path or filter form actions will redirect to home.
  $view->override_path = current_path();
  
  $build = array();
  
  if (user_access('news client full sync')) {
    $build['sync'] = drupal_get_form('news_client_sync_form');
  }
  
  $build['last_cron'] = array(
    '#type' => 'markup',
    '#markup' => '<p>Last cron run: ' . $last_cron_text . '</p>',
  );
  $build['last_update'] = array(
    '#type' => 'markup',
    '#markup' => '<p>Last import/update: ' . $last_update_text . '</p>',
  );
  
  if (user_access('news client fetch updates')) {
    $build['fetch_updates'] = drupal_get_form('news_client_fetch_updates_form');
  }
  
  $build['news_client_cache_view'] = array(
    '#type' => 'markup',
    '#markup' => $view->render('default'),
  );
  
  return $build;
}

/**
 * Sync all articles form.
 */
function news_client_sync_form($form, &$form_state) {
  $form = array();

  $form['sync'] = array(
    '#type' => 'fieldset',
    '#title' => t('Sync All Articles'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  
  $form['sync']['description'] = array(
    '#type' => 'markup',
    '#markup' => '<p>' . 
      t('This will DELETE all articles from the cache not currently defined on the service for the sites defined in settings, and update all other articles.') .
      '</p>',
  );

  $form['sync']['actions'] = array(
    '#type' => 'actions',
  );

  $form['sync']['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Start Sync'),
  );

  return $form;
}

/**
 * Sync all articles submit callback.
 */
function news_client_sync_form_submit($form, &$form_state) {
  $query = db_select('node', 'n');
  $query->fields('n', array('nid'));
  $query->condition('n.type', 'news_client_cached_article');

  $result = $query->execute();

  $operations = array();

  foreach ($result as $row) {
    $operations[] = array('news_client_batch_sync_cached_article', array($row->nid));
  }

  $sync_batch = array(
    'title' => t('Syncing Articles'),
    'operations' => $operations,
    'finished' => 'news_client_batch_sync_finished',
    'init_message' => t('Sync is starting...'),
    'progress_message' => t('@current out of @total. @remaining remaining. (@percentage%)'),
    'error_message' => t('Sync has encountered an error'),
    'file' => drupal_get_path('module', 'news_client') . '/includes/sync.batch.inc',
  );

  batch_set($sync_batch);
  
  $site_identifiers = variable_get('news_client_site_identifiers');
  $article_page = _news_client_fetch_article_page(0, 5, $site_identifiers, '', 1);
  
  if (!is_null($article_page) && !empty($article_page)) {
    $total_articles = $article_page['info']['total_articles'];
    $total_pages = $article_page['info']['total_pages'];
  
    $operations = array();
  
    for($i = 0; $i < $total_pages; $i++) {
      $operations[] = array('news_client_batch_import_process_page', array($i, 5, $site_identifiers, ''));
    }
  
    $import_batch = array(
      'title' => t('Importing Articles'),
      'operations' => $operations,
      'finished' => 'news_client_batch_import_finished',
      'init_message' => t('Import is starting...'),
      'progress_message' => t('@current out of @total. @remaining remaining. (@percentage%)'),
      'error_message' => t('Import has encountered an error'),
      'file' => drupal_get_path('module', 'news_client') . '/includes/import.batch.inc',
    );
  
    batch_set($import_batch);
  }
  else {
    drupal_set_message(t('Import failed to get article info from service.'), 'error');
  }
}

/**
 * Fetch updates form.
 */
function news_client_fetch_updates_form($form, &$form_state) {
  $form = array();
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Fetch Updates'),
  );
  
  return $form;
}

/**
 * Fetch updates form submit callback.
 */
function news_client_fetch_updates_form_submit($form, &$form_state) {
  $updates = _news_client_fetch_article_updates();
  
  if (!is_null($updates)) {
    if (empty($updates)) {
      drupal_set_message(t('There are no updates on the service.'));
    }
    else {
      $operations = array();
      
      foreach ($updates as $update) {
        $operations[] = array('news_client_batch_update_process_update', array($update));
      }
      
      $update_batch = array(
        'title' => t('Updating Articles'),
        'operations' => $operations,
        'finished' => 'news_client_batch_update_finished',
        'init_message' => t('Update is starting...'),
        'progress_message' => t('@current out of @total. @remaining remaining. (@percentage%)'),
        'error_message' => t('Update has encountered an error'),
        'file' => drupal_get_path('module', 'news_client') . '/includes/update.batch.inc',
      );
      
      batch_set($update_batch);
    }
  }
  else {
    drupal_set_message('Update failed to get update info from service.');
  }
}
