<?php
/**
 * Import functions for news_client.
 */

/**
 * Page callback for admin/config/services/news_client/import.
 */
function news_client_import_page() {
  $build = array();
  
  $build['import_by_site'] = drupal_get_form('news_client_import_by_site_form');
  $build['import_single'] = drupal_get_form('news_client_import_single_form');
  
  return $build;
}

/**
 * Import by site form.
 */
function news_client_import_by_site_form($form, &$form_state) {
  $sites_list = _news_client_fetch_sites_list();
  $site_identifiers = variable_get('news_client_site_identifiers');
  
  $site_ids = explode(',', $site_identifiers);
  $site_ids = array_map('trim', $site_ids);
  $site_ids = array_map('intval', $site_ids);
  
  $form = array();
  
  $form['by_site'] = array(
    '#type' => 'fieldset',
    '#title' => t('Import/Update Articles'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  
  $site_options = array();
  
  foreach ($site_ids as $site_id) {
    if (isset($sites_list[$site_id])) {
      $site_options[$site_id] = $sites_list[$site_id]['name'];
    }
  }
  
  $form['by_site']['sites'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Import From'),
    '#description' => t('Select the sites from which to import/update articles.'),
    '#options' => $site_options,
  );
  
  $form['by_site']['actions'] = array(
    '#type' => 'actions',
  );
  
  $form['by_site']['actions'] = array(
    '#type' => 'submit',
    '#value' => t('Import'),
  );
  
  return $form;
}

/**
 * Import by site form submit callback.
 */
function news_client_import_by_site_form_submit($form, &$form_state) {
  $sites = $form_state['values']['sites'];
  
  $selected_sites = array();
  
  foreach ($sites as $site) {
    if ($site != 0) {
      $selected_sites[] = $site;
    }
  }
  
  $sites_query_var = implode(',', $selected_sites);
  
  $article_page = _news_client_fetch_article_page(0, 5, $sites_query_var, '', 1);
  
  if (!is_null($article_page) && !empty($article_page)) {
    $total_articles = $article_page['info']['total_articles'];
    $total_pages = $article_page['info']['total_pages'];
  
    $operations = array();
  
    for($i = 0; $i < $total_pages; $i++) {
      $operations[] = array('news_client_batch_import_process_page', array($i, 5, $sites_query_var, ''));
    }
  
    $batch = array(
      'title' => t('Importing Articles'),
      'operations' => $operations,
      'finished' => 'news_client_batch_import_finished',
      'init_message' => t('Import is starting...'),
      'progress_message' => t('@current out of @total. @remaining remaining. (@percentage%)'),
      'error_message' => t('Import has encountered an error'),
      'file' => drupal_get_path('module', 'news_client') . '/includes/import.batch.inc',
    );
  
    batch_set($batch);
  }
  else {
    drupal_set_message(t('Failed to get article info from service.'), 'error');
  }
}

/**
 * Import single form.
 */
function news_client_import_single_form($form, &$form_state) {
  $form = array();
  
  $form['import_single'] = array(
    '#type' => 'fieldset',
    '#title' => t('Import/Update Single Article'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  
  $form['import_single']['news_service_nid'] = array(
    '#type' => 'textfield',
    '#title' => t('News Service NID'),
    '#description' => t('Nid of the article on the news service.'),
    '#size' => 20,
    '#required' => TRUE,
  );
  
  $form['import_single']['actions'] = array(
    '#type' => 'actions',
  );
  
  $form['import_single']['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Import Single Article'),
  );
  
  return $form;
}

/**
 * Import single form submit callback.
 */
function news_client_import_single_form_submit($form, &$form_state) {
  $news_service_nid = $form_state['values']['news_service_nid'];
  $site_identifiers = variable_get('news_client_site_identifiers');
  
  $article = _news_client_fetch_article($news_service_nid, $site_identifiers);
  
  if (!is_null($article) && !empty($article)) {
    $entity = _news_client_cached_article_populate($article);
    drupal_set_message(t('Article %title was imported from news service.', array('%title' => $entity->title)));
  }
  else {
    drupal_set_message(t('Fetching article from news service endpoint failed.'), 'error');
    if (empty($article)) {
      drupal_set_message('Response from news service was empty. (Article may exist on service, just not for the defined sites on the client.)', 'warning');
    }
  }
}
