<?php
/**
 * Purge functions for news_client.
 */

/**
 * Page callback for admin/config/services/news_client/purge.
 */
function news_client_purge_page() {
  $build = array();
  
  $build['purge_form'] = drupal_get_form('news_client_purge_form');
  
  return $build;
}

/**
 * Purge form.
 */
function news_client_purge_form($form, &$form_state) {
  $form = array();
  
  if (!isset($form_state['storage']['confirm'])) {
    $form['description'] = array(
      '#type' => 'markup',
      '#markup' => '<h2>' . t('Purge ALL Cached Article/Taxonomies.') . '</h2><p>WARNING: This will DELETE all cached articles and taxonomies.</p>',
    );
    
    $form['actions'] = array(
      '#type' => 'actions',
    );
    
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Purge ALL Cached Articles/Taxonomies'),
    );
    
    return $form;
  }
  else {
    return confirm_form($form, t('Purge All Articles/Taxonomies'), array('path' => 'admin/config/services/news_client/cache'), t('This will DELETE ALL cached articles and taxonomies. This cannot be undone.'));
  }
}

/**
 * Purge form submit callback.
 */
function news_client_purge_form_submit($form, &$form_state) {
  if (!isset($form_state['storage']['confirm'])) {
    $form_state['storage']['confirm'] = TRUE;
    $form_state['rebuild'] = TRUE;
  }
  else {
    // Delete all nodes of type news_client_cached_article.
    $cached_article_query = new EntityFieldQuery();
    $cached_article_query->entityCondition('entity_type', 'node');
    $cached_article_query->entityCondition('bundle', 'news_client_cached_article');
    
    $result = $cached_article_query->execute();
    
    if (isset($result['node'])) {
      // Delete all cached article nodes.
      $cached_article_nids = array_keys($result['node']);
      node_delete_multiple($cached_article_nids);
    }
    
    // Delete all terms in news_client_sites and news_client_tags.
    $sites_vocabulary = taxonomy_vocabulary_machine_name_load('news_client_sites');
    foreach (taxonomy_get_tree($sites_vocabulary->vid) as $term) {
      taxonomy_term_delete($term->tid);
    }
    
    $tags_vocabulary = taxonomy_vocabulary_machine_name_load('news_client_tags');
    foreach (taxonomy_get_tree($tags_vocabulary->vid) as $term) {
      taxonomy_term_delete($term->tid);
    }
    
    // Reset last update time.
    variable_set('news_client_last_update', 0);
    
    drupal_set_message(t('Cached News Articles and Taxonomies Purged.'));
    
    $form_state['redirect'] = 'admin/config/services/news_client/cache';
  }
}
