(function(factory) {
    if (typeof define === 'function' && define.amd) {
        define(['jquery'], factory);
    } else if (typeof exports !== 'undefined') {
        module.exports = factory(require('jquery'));
    } else {
        factory(jQuery);
    }

}(function($) {


    function carousel_resize() {
        $(".carousel").each(function() {
            var $this = $(this),
                $largest = 0,
                $carousel_container = $this.find('.carousel--container');

            $this.find(".carousel--content").each(function(){
                var $height = $(this).height();
                if ($height > $largest) {
                    $largest = $height;
                }
            });

            $carousel_container.css('height', $largest + 'px');
        });
    }

    $.fn.extend({
        accessHide:  function (){
            $(this).attr("aria-hidden", "true").find('a').attr('tabindex', '-1');
        },
        accessShow:  function (){
            $(this).removeAttr("aria-hidden").find('a').attr('tabindex', '0');
        }
    });

    $(function() {
        /*
         * jQuery Accessible carousel System, using ARIA
         * @version v1.7.1
         * Website: https://a11y.nicolas-hoffmann.net/carousel/
         * License MIT: https://github.com/nico3333fr/jquery-accessible-carousel-aria/blob/master/LICENSE
         */
        /* loading aria carousel ----------------------------------------------------------------------------------------------------------------------- */
        var $carousel_container = $('.carousel--container'),
            $carousel_content = $('.carousel--content'),
            $body = $('body');

        var $pause = false;

        if ($carousel_container.length && $carousel_content.length) { // if there are at least one content and one container :)

            var hash = window.location.hash.replace("#", ""),
                getTransEndEventName = function() {
                    var i,
                        el = document.createElement('div'),
                        transitions = {
                            'WebkitTransition': 'webkitTransitionEnd',
                            'MozTransition': 'transitionend',
                            'OTransition': 'oTransitionEnd otransitionend',
                            'msTransition': 'MSTransitionEnd',
                            'transition': 'transitionend',
                            'webkitAnimation': 'webkitAnimationEnd',
                            'MozAnimation': 'animationend',
                            'msAnimation': 'msAnimationEnd',
                            'animation': 'animationend'

                        };

                    for (i in transitions) {
                        if (transitions.hasOwnProperty(i) && typeof el.style[i] !== "undefined") {
                            return transitions[i];
                        }
                    }

                    return "NextSlide"; // CSS transitions not supported
                },
                transEndEventName = getTransEndEventName();
            // Do something when the transition/animation ends
            $body.on(transEndEventName, ".carousel--content", function() {
                var $this = $(this),
                    $parent = $this.parents(".carousel");
            });

            $carousel_container.each(function(index_carousel_container) {
                var $this_carousel_container = $(this),
                    options = $this_carousel_container.data(),
                    $index_carousel = index_carousel_container + 1,
                    $carousel_prefix_classes = options.carouselPrefixClasses + '-' || '',
                    $carousel_span_text_class = options.carouselSpanTextClass || '',
                    $carousel_span_text = options.carouselSpanText || '',
                    $carousel_span_text_final = '',
                    $carousel_transition = options.carouselTransition || '',
                    $carousel_btn_previous_img = options.carouselBtnPreviousImg || '',
                    $carousel_btn_next_img = options.carouselBtnNextImg || '',
                    $carousel_btn_previous_text = options.carouselBtnPreviousText || '',
                    $carousel_btn_previous_textflat = $($carousel_btn_previous_text).text() || $carousel_btn_previous_text,
                    $carousel_btn_next_text = options.carouselBtnNextText || '',
                    $carousel_btn_next_textflat = $($carousel_btn_next_text).text() || $carousel_btn_next_text,
                    $carousel_hx = options.carouselHx || '',
                    $carousel_existing_hx = options.carouselExistingHx || '';

                if ($carousel_transition != "") {
                    $this_carousel_container.addClass($carousel_transition);
                }

                // information to know which tab is activated
                $this_carousel_container.addClass('carouselslide-' + $index_carousel + '-1')
                    .addClass($carousel_prefix_classes + 'carousel--container')
                    .addClass($carousel_prefix_classes + 'carousel--container--' + $index_carousel);


                /* insert list before carousel--container  -------------------------------------------------------------------------- */
                var navigation = '<ol class="js-carousel--control--list ' + $carousel_prefix_classes + 'carousel--control--list ' + $carousel_prefix_classes + 'carousel--control--list--' + $index_carousel + '" role="tablist">';

                $this_carousel_container.find(".carousel--content").each(function(index) {
                    var $this = $(this),
                        $index_readable = index + 1,
                        $content_id = "id_carousel_content_" + $index_carousel + "_" + $index_readable;

                    // add attributes
                    $this.attr({
                            "role": "tabpanel",
                            "id": $content_id,
                            "aria-labelledby": "label_" + $content_id
                        }).accessHide();

                    // focusable hx
                    if ($carousel_existing_hx != '') {
                        $this.find($carousel_existing_hx);
                        $carousel_span_text_final =  $carousel_span_text + ' ' + $index_readable + ': ' + $this.find($carousel_existing_hx).text();
                    } else {
                        $this.prepend('<' + $carousel_hx + ' class="element-invisible" tabindex="0">' + $carousel_span_text + ' ' + $index_readable + '</' + $carousel_hx + '>');
                        $carousel_span_text_final = $carousel_span_text + ' ' + $index_readable;
                    }

                    navigation = navigation + '<li class="js-carousel--control--list--item ' + $carousel_prefix_classes + 'carousel--control--list--item" role="tab" id="label_' + $content_id + '" aria-controls="' + $content_id + '" tabindex="-1" aria-selected="false">';

                    navigation = navigation + '<span class="' + $carousel_prefix_classes + 'carousel--control--list--text';
                    if ($carousel_span_text_class !== '') {
                        navigation = navigation + ' ' + $carousel_span_text_class;
                    }
                    navigation = navigation + '">' + $carousel_span_text_final + '</span></li>';
                });
                navigation = navigation + '</ol>';

                $(navigation).insertBefore($this_carousel_container);


                /* Add previous/next buttons ----------------------------------------------------------------------------------------------------- */
                if ($carousel_btn_previous_text !== '') {

                    var previous_button = '<div class="js-carousel--button-container element-invisible ' + $carousel_prefix_classes + 'carousel--button-container js-carousel--button--previous ' + $carousel_prefix_classes + 'carousel--button--previous ' + $carousel_prefix_classes + 'carousel--button--previous--' + $index_carousel + '"><button type="button" title="' + $carousel_btn_previous_textflat + '" class="js-carousel--button--button ' + $carousel_prefix_classes + 'carousel--button--button">';
                    if ($carousel_btn_previous_img !== '') {
                        previous_button = previous_button + '<img src="' + $carousel_btn_previous_img + '" alt="' + $carousel_btn_previous_textflat + '" class="carousel--button--img" />';
                    } else {
                        previous_button = previous_button + $carousel_btn_previous_text;
                    }
                    previous_button = previous_button + '</button>';
                    $(previous_button).insertBefore($this_carousel_container);

                }


                if ($carousel_btn_next_text !== '') {

                    var next_button = '<div class="js-carousel--button-container element-invisible ' + $carousel_prefix_classes + 'carousel--button-container js-carousel--button--next ' + $carousel_prefix_classes + 'carousel--button--next ' + $carousel_prefix_classes + 'carousel--button--next--' + $index_carousel + '"><button type="button" title="' + $carousel_btn_next_textflat + '" class="js-carousel--button--button ' + $carousel_prefix_classes + 'carousel--button--button">';
                    if ($carousel_btn_next_img !== '') {
                        next_button = next_button + '<img src="' + $carousel_btn_next_img + '" alt="' + $carousel_btn_next_textflat + '" class="carousel--button--img" />';
                    } else {
                        next_button = next_button + $carousel_btn_next_text;
                    }
                    next_button = next_button + '</button>';

                    $(next_button).insertAfter($this_carousel_container);
                }

            });


            // check hash concerns carousel
            if (hash !== "") {
                if ($("#" + hash + ".carousel--content").length === 0) {
                    hash = '';
                }
            }

            /* then check defaults ---------------------------------------------------------------------------------------------------------- */
            if (hash !== "" && $("#" + hash + ".carousel--content").length) {

                // display
                $("#" + hash + ".carousel--content").accessShow();

                // selection menu
                $("#label_" + hash + ".js-carousel--control--list--item").attr({
                    "aria-selected": "true",
                    "tabindex": 0
                });

                $carousel_container.trigger('carousel:slide-changed');
                // update of carouselslide-x-x
                $carousel_container = $("#" + hash + ".carousel--content").parents(".carousel--container");


                // get .carouselslide-x-x
                var classes = $carousel_container.attr('class').split(' ');
                var found = false;
                var $previous_content;
                var $new_content;
                var i = 0;
                while (found === false && i < classes.length) {
                    if (classes[i].substr(0, 15) === "carouselslide-") {
                        $previous_content = classes[i];
                        found = true;
                    }
                    i++;
                }

                var $index_tab = ($(".carousel--content").index($("#" + hash + ".carousel--content"))) + 1;
                var tab = $previous_content.split('-');
                $new_content = tab[0] + '-' + tab[1] + '-' + $index_tab;
                // replace .carouselslide-1-x by .carouselslide-1-$index_tab
                $carousel_container.removeClass($previous_content).addClass($new_content).trigger("NextSlide");


            }

            //var $carousels = $(".carousel");
            // active slide
            $(".carousel").each(function() {
                var $this = $(this),
                    $carousel_container = $this.find('.carousel--container'),
                    options = $carousel_container.data(),
                    carousel_active_slide = Number(options.carouselActiveSlide) - 1 || 0,
                    $control_list_links = $this.find('.js-carousel--control--list--item');

                // if there is an active slide, in a correct range and not already a valid hash for this carousel
                if (carousel_active_slide !== 0 && carousel_active_slide < $control_list_links.length && $this.find(".js-carousel--control--list--item[aria-selected=true]").length === 0) {
                    var $control_list_link = $control_list_links.eq(carousel_active_slide),
                        $content_linked = $('#' + $control_list_link.attr('aria-controls'));

                    $control_list_links.eq(carousel_active_slide).attr({
                        "aria-selected": "true",
                        "tabindex": 0
                    });

                    $content_linked.accessShow();

                    // get .carouselslide-x-x
                    var classes = $carousel_container.attr('class').split(' ');
                    var found = false;
                    var previous_content;
                    var new_content;
                    var i = 0;
                    while (found === false && i < classes.length) {
                        if (classes[i].substr(0, 15) === "carouselslide-") {
                            previous_content = classes[i];
                            found = true;
                        }
                        i++;
                    }

                    var tab = previous_content.split('-');
                    new_content = tab[0] + '-' + tab[1] + '-' + (carousel_active_slide + 1);
                    // replace .carouselslide-1-x by .carouselslide-1-$index_tab
                    $carousel_container.removeClass(previous_content).addClass(new_content).trigger("NextSlide");

                }

            });

            // if no selected => select first
            $(".carousel").each(function() {
                var $this = $(this),
                    $first_content = $this.find(".carousel--content:first");
                if ($this.find(".js-carousel--control--list--item[aria-selected=true]").length === 0) {
                    $this.find(".js-carousel--control--list--item:first").attr({
                        "aria-selected": "true",
                        "tabindex": 0
                    });
                    $carousel_container.trigger('carousel:slide-changed');
                    $first_content.accessShow();
                }
            });

        }

        /* Events ---------------------------------------------------------------------------------------------------------- */
        /* click on a tab link */
        $body.on("click", ".js-carousel--control--list--item", function(event, additionnal) {
                /* eslint-disable indent */
                var $this = $(this),
                    $parent = $this.parents(".carousel"),
                    $tab_linked = $("#" + $this.attr("aria-controls")),
                    $index_tab,
                    $previous_content,
                    $carousel_container = $parent.find(".carousel--container"),
                    $carousel_hx = $carousel_container.attr('data-carousel-hx'),
                    $carousel_existing_hx = $carousel_container.attr('data-carousel-existing-hx'),
                    $carousel_hx_final = '';

                if (typeof $carousel_hx === "undefined" || $carousel_hx === "undefined" || $carousel_hx === "") {
                    $carousel_hx = '';
                }
                if (typeof $carousel_existing_hx === "undefined" || $carousel_existing_hx === "undefined" && $carousel_existing_hx === "") {
                    $carousel_existing_hx = '';
                }

                if ($carousel_existing_hx != '') {
                    $carousel_hx_final = $carousel_existing_hx;
                } else {
                    $carousel_hx_final = $carousel_hx;
                }

                // get .carousel-slide-1-x
                var classes = $carousel_container.attr('class').split(' ');
                var found = false;
                var i = 0;
                while (found === false && i < classes.length) {
                    if (classes[i].substr(0, 14) === "carouselslide-") {
                        $previous_content = classes[i];
                        found = true;
                    }
                    i++;
                }

                // remove aria selected on all link
                $parent.find(".js-carousel--control--list--item").attr("aria-selected", "false").attr("tabindex", -1);
                // add aria selected on $this
                $this.attr({
                    "aria-selected": "true",
                    "tabindex": 0
                });

                if (event.originalEvent && event.originalEvent.type == 'click') {
                    $this.focus();
                }

                $carousel_container.trigger('carousel:slide-changed');
                // add aria-hidden on all tabs
                $parent.find(".carousel--content").accessHide();

                // remove aria-hidden on tab linked
                $tab_linked.accessShow();

                // find index of clicked tab
                $index_tab = ($parent.find(".carousel--content").index($tab_linked)) + 1;
                var tab = $previous_content.split('-');
                $new_content = tab[0] + '-' + tab[1] + '-' + $index_tab;
                // replace .carouselslide-1-x by .carouselslide-1-$index_tab
                $parent.find(".carousel--container").removeClass($previous_content).addClass($new_content).trigger("NextSlide");

                // if coming from button next/prev, add focus to next/prev content
                if (additionnal == 'next' || additionnal == 'prev') {

                    $parent.find(".carousel--container div").one(
                        "webkitTransitionEnd MSTransitionEnd oTransitionEnd transitionend animationend webkitAnimationEnd oAnimationEnd oanimationend msAnimationEnd animationend",
                        function() {
                            $(this).data("transitioning", false); // Transition has ended.
                            setTimeout(function() {
                                $tab_linked.children($carousel_hx_final).focus();
                            }, 0);
                        }
                    );

                }

                event.preventDefault();
            })
            /* eslint-enable indent */

            /* Key down in tabs */
            .on("keydown", ".carousel", function(event) {

                var $parent = $(this),
                    $activated,
                    $focus_on_tab_only = false;

                // some event should be activated only if the focus is on tabs (not on tabpanel)
                if ($(document.activeElement).is($parent.find('.js-carousel--control--list--item'))) {
                    $focus_on_tab_only = true;
                }

                // catch keyboard event only if focus is on tab
                if ($focus_on_tab_only && !event.ctrlKey) {
                    // strike up or left in the tab
                    if (event.keyCode == 37 || event.keyCode == 38) {
                        // find previous tab
                        $activated = $parent.find('.js-carousel--control--list--item[aria-selected="true"]');
                        // if we are on first => activate last
                        if ($activated.is(".js-carousel--control--list--item:first-child")) {
                            $parent.find(".js-carousel--control--list--item:last-child").click();
                            setTimeout(function() {
                                $parent.find(".js-carousel--control--list--item:last-child").focus();
                            }, 0);
                        }
                        // else activate previous
                        else {
                            $activated.prev(".js-carousel--control--list--item").click();
                            setTimeout(function() {
                                $activated.prev(".js-carousel--control--list--item").focus();
                            }, 0);
                        }
                        event.preventDefault();
                    }
                    // strike down or right in the tab
                    else if (event.keyCode == 40 || event.keyCode == 39) {
                        // find next tab
                        $activated = $parent.find('.js-carousel--control--list--item[aria-selected="true"]');
                        // if we are on last => activate first
                        if ($activated.is(".js-carousel--control--list--item:last-child")) {
                            $parent.find(".js-carousel--control--list--item:first-child").click();
                            setTimeout(function() {
                                $parent.find(".js-carousel--control--list--item:first-child").focus();
                            }, 0);
                        }
                        // else activate next
                        else {
                            $activated.next(".js-carousel--control--list--item").click();
                            setTimeout(function() {
                                $activated.next(".js-carousel--control--list--item").focus();
                            }, 0);
                        }
                        event.preventDefault();
                    } else if (event.keyCode == 36) {
                        // activate first tab
                        $parent.find(".js-carousel--control--list--item:first-child").click();
                        setTimeout(function() {
                            $parent.find(".js-carousel--control--list--item:first-child").focus();
                        }, 0);
                        event.preventDefault();
                    } else if (event.keyCode == 35) {
                        // activate last tab
                        $parent.find(".js-carousel--control--list--item:last-child").click();
                        setTimeout(function() {
                            $parent.find(".js-carousel--control--list--item:last-child").focus();
                        }, 0);
                        event.preventDefault();
                    }

                }

            })
            .on("keydown", ".carousel--content", function(event) {

                var $this = $(this),
                    $tab_to_focus,
                    $parent;

                // CTRL up/Left
                if ((event.keyCode == 37 || event.keyCode == 38) && event.ctrlKey) {
                    $tab_to_focus = $this.attr('aria-labelledby');
                    setTimeout(function() {
                        $("#" + $tab_to_focus).focus();
                    }, 0);
                    event.preventDefault();
                }
                // CTRL PageUp
                if (event.keyCode == 33 && event.ctrlKey) {
                    var $tab_focused = $this.attr('aria-labelledby');
                    setTimeout(function() {
                        $("#" + $tab_focused).focus();
                    }, 0);

                    $parent = $("#" + $tab_focused).parent();

                    // if we are on first => activate last
                    if ($parent.is(".js-carousel--control--list--item:first-child")) {
                        $parent.parent().find(".js-carousel--control--list--item:last-child").click();
                        setTimeout(function() {
                            $parent.parent().find(".js-carousel--control--list--item:last-child").focus();
                        }, 0);
                    }
                    // else activate prev
                    else {
                        $parent.prev(".js-carousel--control--list--item").click();
                        setTimeout(function() {
                            $parent.prev(".js-carousel--control--list--item").focus();
                        }, 0);
                    }
                    event.preventDefault();
                }
                // CTRL PageDown
                if (event.keyCode == 34 && event.ctrlKey) {
                    $tab_focused = $this.attr('aria-labelledby');
                    setTimeout(function() {
                        $("#" + $tab_focused).focus();
                    }, 0);

                    $parent = $("#" + $tab_focused).parent();
                    // if we are on last => activate first
                    if ($parent.is(".js-carousel--control--list--item:last-child")) {
                        $parent.parent().find(".js-carousel--control--list--item:first-child").click();
                        setTimeout(function() {
                            $parent.parent().find(".js-carousel--control--list--item:first-child").focus();
                        }, 0);
                    }
                    // else activate next
                    else {
                        $parent.next(".js-carousel--control--list--item").click();
                        setTimeout(function() {
                            $parent.next(".js-carousel--control--list--item").focus();
                        }, 0);
                    }
                    event.preventDefault();
                }

            })

            .on({
                mouseenter: function () {
                    $pause = true;
                },
                mouseleave: function () {
                    $pause = false;
                }
            }, ".carousel");

        /* click on a button prev/next */
        // prev
        $body.on("click", ".js-carousel--button--previous button", function(event) {

                /* eslint-disable indent */
                event.preventDefault();

                var $this = $(this),
                    $activated,
                    $parent = $this.parents(".carousel");

                // find previous tab
                $activated = $parent.find('.js-carousel--control--list--item[aria-selected="true"]');
                // if we are on first => activate last

                if ($activated.is(".js-carousel--control--list--item:first-child")) {
                    $parent.find(".js-carousel--control--list--item:last-child").trigger('click', 'prev');
                }
                // else activate previous
                else {
                    $activated.prev(".js-carousel--control--list--item").trigger('click', 'prev');
                }


            })
            /* eslint-enable indent */

            // next
            .on("click", ".js-carousel--button--next button", function(event) {

                event.preventDefault();

                var $this = $(this),
                    $activated,
                    $parent = $this.parents(".carousel");

                // find next tab
                $activated = $parent.find('.js-carousel--control--list--item[aria-selected="true"]');
                // if we are on last => activate first
                    var e = jQuery.Event("keydown");
                e.which = 39; // # Some key code value

                if ($activated.is(".js-carousel--control--list--item:last-child")) {
                    $parent.find(".js-carousel--control--list--item:first-child").trigger('click', 'next');
                }
                // else activate next
                else {
                    $activated.next(".js-carousel--control--list--item").trigger('click', 'next');
                }

            });

        $carousel_container.trigger('carousel:initialized');

        carousel_resize();
        /*
        var tid = setInterval(autoPlay, 6000);

        function autoPlay() {
          if ($pause != true && $('.carousel').find(':focus').length == 0) {
            $(".js-carousel--button--next button").click();
          }
        }

        $('body').prepend('<button class="carousel-pause-button">pause</button>');
        */
        $(window).on('load resize', function() {
            carousel_resize();
        });
    });
}));
